library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity FFT64 is
  port (
    RESET     : in  std_logic;
    CLK       : in  std_logic;
    HEAD    : in  std_logic;                         -- <1,1,u>
    FFTIN_I    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    FFTIN_Q    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    OUTHEAD : out std_logic;                         -- <1,1,u>
    FFTOUT_I   : out std_logic_vector(13 downto 0);  -- <14,6,t>
    FFTOUT_Q   : out std_logic_vector(13 downto 0)); -- <14,6,t>
end;

architecture RTL of FFT64 is

component STAGE1 is
  port (
    RESET     : in  std_logic;
    CLK       : in  std_logic;
    S1HEAD    : in  std_logic;                      -- <1,1,u>
    S1IN_I    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    S1IN_Q    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    S1OUTHEAD : out std_logic;                      -- <1,1,u>
    S1OUT_I   : out std_logic_vector(13 downto 0);  -- <14,6,t>
    S1OUT_Q   : out std_logic_vector(13 downto 0)); -- <14,6,t>     
end component;

component STAGE2
  port (
    RESET     : in  std_logic;
    CLK       : in  std_logic;
    S2HEAD    : in  std_logic;                      -- <1,1,u>
    S2IN_I    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    S2IN_Q    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    S2OUTHEAD : out std_logic;                      -- <1,1,u>
    S2OUT_I   : out std_logic_vector(13 downto 0);  -- <14,6,t>
    S2OUT_Q   : out std_logic_vector(13 downto 0)); -- <14,6,t>
end component;

component STAGE3
  port (
    RESET     : in  std_logic;
    CLK       : in  std_logic;
    S3HEAD    : in  std_logic;                      -- <1,1,u>
    S3IN_I    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    S3IN_Q    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    S3OUTHEAD : out std_logic;                      -- <1,1,u>
    S3OUT_I   : out std_logic_vector(13 downto 0);  -- <14,6,t>
    S3OUT_Q   : out std_logic_vector(13 downto 0)); -- <14,6,t>
end component;

component REORDER
  port (
    RESET     : in  std_logic;
    CLK       : in  std_logic;
    ROHEAD    : in  std_logic;                      -- <1,1,u>
    ROIN_I    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    ROIN_Q    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    ROOUTHEAD : out std_logic;                      -- <1,1,u>
    ROOUT_I   : out std_logic_vector(13 downto 0);  -- <14,6,t>
    ROOUT_Q   : out std_logic_vector(13 downto 0)); -- <14,6,t>
end component;

signal  S1HEAD, S1OUTHEAD  : std_logic; 
signal  S2HEAD, S2OUTHEAD  : std_logic; 
signal  S3HEAD, S3OUTHEAD  : std_logic; 
signal  ROHEAD, ROOUTHEAD  : std_logic;  
signal S1IN_I,  S1IN_Q  : std_logic_vector (13 downto 0); -- <14,6,t> 
signal S1OUT_I, S1OUT_Q : std_logic_vector (13 downto 0); -- <14,6,t> 
signal S2IN_I,  S2IN_Q  : std_logic_vector (13 downto 0); -- <14,6,t> 
signal S2OUT_I, S2OUT_Q : std_logic_vector (13 downto 0); -- <14,6,t> 
signal S3IN_I,  S3IN_Q  : std_logic_vector (13 downto 0); -- <14,6,t> 
signal S3OUT_I, S3OUT_Q : std_logic_vector (13 downto 0); -- <14,6,t> 
signal ROIN_I,  ROIN_Q  : std_logic_vector (13 downto 0); -- <14,6,t> 
signal ROOUT_I, ROOUT_Q : std_logic_vector (13 downto 0); -- <14,6,t> 

begin

S1HEAD <= HEAD; S1IN_I <= FFTIN_I; S1IN_Q <= FFTIN_Q;

ST1: STAGE1 port map (RESET, CLK, S1HEAD, S1IN_I, S1IN_Q, S1OUTHEAD, S1OUT_I, S1OUT_Q);

S2HEAD <= S1OUTHEAD; S2IN_I <= S1OUT_I; S2IN_Q <= S1OUT_Q;

ST2: STAGE2 port map (RESET, CLK, S2HEAD, S2IN_I, S2IN_Q, S2OUTHEAD, S2OUT_I, S2OUT_Q);

S3HEAD <= S2OUTHEAD; S3IN_I <= S2OUT_I; S3IN_Q <= S2OUT_Q;

ST3: STAGE3 port map (RESET, CLK, S3HEAD, S3IN_I, S3IN_Q, S3OUTHEAD, S3OUT_I, S3OUT_Q);

ROHEAD <= S3OUTHEAD; ROIN_I <= S3OUT_I; ROIN_Q <= S3OUT_Q;

RO0: REORDER port map (RESET, CLK, ROHEAD, ROIN_I, ROIN_Q, ROOUTHEAD, ROOUT_I, ROOUT_Q);

OUTHEAD <= ROOUTHEAD; FFTOUT_I <= ROOUT_I; FFTOUT_Q <= ROOUT_Q;

end;