library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity STAGE3 is
  port (
    RESET     : in  std_logic;
    CLK       : in  std_logic;
    S3HEAD    : in  std_logic;                      -- <1,1,u>
    S3IN_I    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    S3IN_Q    : in  std_logic_vector(13 downto 0);  -- <14,6,t>
    S3OUTHEAD : out std_logic;                      -- <1,1,u>
    S3OUT_I   : out std_logic_vector(13 downto 0);  -- <14,6,t>
    S3OUT_Q   : out std_logic_vector(13 downto 0)); -- <14,6,t>
end;

architecture RTL of STAGE3 is

signal  HEAD  : std_logic_vector(4 downto 0);  -- 5 FFs for Head Signal Delay
signal  COUNT : std_logic_vector(5 downto 0);  -- count 0 to 63 cycles
signal  PHASE : std_logic_vector(1 downto 0);  -- PHASE 0 to 3
signal  S1, S2, S3 : std_logic;

signal REGIN_I : std_logic_vector (13 downto 0); -- <14,6,t>
signal REGIN_Q : std_logic_vector (13 downto 0); -- <14,6,t>
type shiftreg7 is array (0 to 6) of std_logic_vector (13 downto 0); -- <14,6,t> 
signal REG_I  : shiftreg7;
signal REG_Q  : shiftreg7;
signal A_I, A_Q : std_logic_vector (13 downto 0); -- <14,6,t> 
signal B_I, B_Q : std_logic_vector (13 downto 0); -- <14,6,t> 
signal C_I, C_Q : std_logic_vector (13 downto 0); -- <14,6,t> 
signal D_I, D_Q : std_logic_vector (13 downto 0); -- <14,6,t> 
signal Y_I, Y_Q : std_logic_vector (13 downto 0); -- <14,6,t>  butterfly output

begin

-- OUTHEAD GENERATION
    process ( CLK, RESET ) begin 
      if (RESET = '1') then
          HEAD(0) <= '0';
          HEAD(1) <= '0';
          HEAD(2) <= '0';
          HEAD(3) <= '0';
          HEAD(4) <= '0';
      elsif ( CLK'event and CLK = '1' ) then
            HEAD(0) <= S3HEAD;
            HEAD(1) <= HEAD(0);
            HEAD(2) <= HEAD(1);
            HEAD(3) <= HEAD(2);
            HEAD(4) <= HEAD(3);
      end if; 
  end process;
    S3OUTHEAD <= HEAD(4);

-- 64 CYCLE COUNTER
    process ( CLK, RESET ) begin 
    if (RESET = '1') then
        COUNT <= "000000";
    elsif ( CLK'event and CLK = '1' ) then
        if (S3HEAD = '1') then 
            COUNT <= "000000"; 
        else
            COUNT <= unsigned(COUNT) + '1';
        end if;
    end if; 
    end process;

-- PHASE GENERATOR
    PHASE <= COUNT(1 downto 0);

-- SELECT SIGNAL GENERATOR
    S1 <= PHASE(0) or PHASE(1);
    S2 <= PHASE(1);
    S3 <= PHASE(0) and PHASE(1);

-- SHIFT REGISTER 
    process ( CLK ) begin 
      if ( CLK'event and CLK = '1' ) then
        REGIN_I  <=  S3IN_I;    REGIN_Q  <=  S3IN_Q;
        REG_I(0) <=  REGIN_I;   REG_Q(0) <=  REGIN_Q; 
        REG_I(1) <=  REG_I(0);  REG_Q(1) <=  REG_Q(0);
        REG_I(2) <=  REG_I(1);  REG_Q(2) <=  REG_Q(1);
        REG_I(3) <=  REG_I(2);  REG_Q(3) <=  REG_Q(2);
        REG_I(4) <=  REG_I(3);  REG_Q(4) <=  REG_Q(3);
        REG_I(5) <=  REG_I(4);  REG_Q(5) <=  REG_Q(4);
        REG_I(6) <=  REG_I(5);  REG_Q(6) <=  REG_Q(5);
      end if; 
    end process;

-- A,B,C,D GENERATION
    A_I <= REG_I(3);
    A_Q <= REG_Q(3);
    B_I <= REG_I(0) when S1 = '0' else REG_I(4);
    B_Q <= REG_Q(0) when S1 = '0' else REG_Q(4);
    C_I <= REG_I(1) when S2 = '0' else REG_I(5);
    C_Q <= REG_Q(1) when S2 = '0' else REG_Q(5);
    D_I <= REG_I(2) when S3 = '0' else REG_I(6);
    D_Q <= REG_Q(2) when S3 = '0' else REG_Q(6);

-- RADIX-4 BUTTERFLY 

process (PHASE, A_I, B_I, C_I, D_I, A_Q, B_Q, C_Q, D_Q) begin
  case PHASE is
    when "00" => 
       Y_I <= signed(A_I) + signed(B_I) + signed(C_I) + signed (D_I);
       Y_Q <= signed(A_Q) + signed(B_Q) + signed(C_Q) + signed (D_Q);
    when "01" => 
       Y_I <= signed(A_Q) + signed(B_I) - signed(C_Q) - signed (D_I);
       Y_Q <= - signed(A_I) + signed(B_Q) + signed(C_I) - signed (D_Q);
    when "10" => 
       Y_I <= signed(A_I) - signed(B_I) + signed(C_I) - signed (D_I);
       Y_Q <= signed(A_Q) - signed(B_Q) + signed(C_Q) - signed (D_Q);
    when "11" => 
       Y_I <= signed(A_Q) - signed(B_I) - signed(C_Q) + signed (D_I);
       Y_Q <= - signed(A_I) - signed(B_Q) + signed(C_I) + signed (D_Q);
    when others => 
       Y_I <= (others => 'X');
       Y_Q <= (others => 'X');
  end case;
end process;
S3OUT_I <= Y_I;
S3OUT_Q <= Y_Q;

end;